// DEMO8.CPP -- demonstrate stream functions
// Jim Fawcett, CSE687

// (1) - include declarations for all file I/O functions
#include <fstream.h>    // fstream includes iostream

//----< process_file(istream& istrm) >-----------------------//
//      demonstrate various stream functions

void process_file(ifstream& istrm) {

// (2a) - part of rewind: mark beginning of file
  long position = istrm.tellg();

// (3a) - read file line by line: line is string of characters
//                                terminated with '\n'
  #define MAXSIZE 512
  char buffer[MAXSIZE+1];
  while(istrm) {
    istrm.getline(buffer,MAXSIZE);
    cout << buffer << "\n";
  // (3b) - check for end of file: needed to terminate while loop
  //                               only if last line doesn't have
  //                               a newline character, '\n'
    if(istrm.eof()) {      // will only get here if getline fails
      cout << "\n*** at file end ***\n\n";
      break;    // break out of while loop
    }
  }

// (2b) - rewind file to position
  istrm.seekg(position);     // return to initial position
  istrm.clear();             // clear eof state

// (4) - read file char by char
  char ch;
  while(istrm.get(ch))
    cout << ch;
  if(istrm.eof())     // not needed to terminate while loop
    cout << "\n*** at file end again ***\n";
}

//----< int main(int argc, char *argv[]) >-------------------//
//      Open files named on command line.
//      Does not expand wild card characters.

int main(int argc, char *argv[]) {
			// argv[i] points to ith token
  if (argc < 2) {       // argc counts number of tokens
    cerr << "please enter file name on command line\n";
    return 1;
  }
  int file_count = 0;
  while (--argc) {
    ifstream istrm(argv[++file_count]);
    if (!istrm) {       // istrm will be null if can't open file
      cerr << "  can't open  " << argv[file_count] << "\n";
      continue;   // causes processing to skip to bottom of loop
    }
    process_file(istrm);
    istrm.close();
  }
  return 0;
}
